@@ -20,6 +20,8 @@ context("To/from Python")
 test_that("install_pyarrow", {
   skip_on_cran()
   skip_if_not_dev_mode()
+  # Python problems on Apple M1 still
+  skip_if(grepl("arm-apple|aarch64.*darwin", R.Version()$platform))
   skip_if_not_installed("reticulate")
   venv <- try(reticulate::virtualenv_create("arrow-test"))
   # Bail out if virtualenv isn't available
@@ -41,7 +43,7 @@ test_that("Array to Python", {
   pa <- reticulate::import("pyarrow", convert = FALSE)
   r <- Array$create(c(1, 2, 3))
   py <- pa$concat_arrays(list(r))
-  expect_is(py, "pyarrow.lib.Array")
+  expect_s3_class(py, "pyarrow.lib.Array")
   expect_equal(reticulate::py_to_r(py), r)
 })
 
@@ -50,7 +52,7 @@ test_that("RecordBatch to/from Python", {
   pa <- reticulate::import("pyarrow", convert = FALSE)
   batch <- record_batch(col1 = c(1, 2, 3), col2 = letters[1:3])
   py <- reticulate::r_to_py(batch)
-  expect_is(py, "pyarrow.lib.RecordBatch")
+  expect_s3_class(py, "pyarrow.lib.RecordBatch")
   expect_equal(reticulate::py_to_r(py), batch)
 })
 
@@ -61,8 +63,8 @@ test_that("Table and ChunkedArray from Python", {
   tab <- Table$create(batch, batch)
   pybatch <- reticulate::r_to_py(batch)
   pytab <- pa$Table$from_batches(list(pybatch, pybatch))
-  expect_is(pytab, "pyarrow.lib.Table")
-  expect_is(pytab[0], "pyarrow.lib.ChunkedArray")
+  expect_s3_class(pytab, "pyarrow.lib.Table")
+  expect_s3_class(pytab[0], "pyarrow.lib.ChunkedArray")
   expect_equal(reticulate::py_to_r(pytab[0]), tab$col1)
   expect_equal(reticulate::py_to_r(pytab), tab)
 })
@@ -73,11 +75,11 @@ test_that("Table and ChunkedArray to Python", {
   tab <- Table$create(batch, batch)
 
   pychunked <- reticulate::r_to_py(tab$col1)
-  expect_is(pychunked, "pyarrow.lib.ChunkedArray")
+  expect_s3_class(pychunked, "pyarrow.lib.ChunkedArray")
   expect_equal(reticulate::py_to_r(pychunked), tab$col1)
 
   pytab <- reticulate::r_to_py(tab)
-  expect_is(pytab, "pyarrow.lib.Table")
+  expect_s3_class(pytab, "pyarrow.lib.Table")
   expect_equal(reticulate::py_to_r(pytab), tab)
 })
 
@@ -85,7 +87,7 @@ test_that("RecordBatch with metadata roundtrip", {
   skip_if_no_pyarrow()
   batch <- RecordBatch$create(example_with_times)
   pybatch <- reticulate::r_to_py(batch)
-  expect_is(pybatch, "pyarrow.lib.RecordBatch")
+  expect_s3_class(pybatch, "pyarrow.lib.RecordBatch")
   expect_equal(reticulate::py_to_r(pybatch), batch)
   expect_identical(as.data.frame(reticulate::py_to_r(pybatch)), example_with_times)
 })
@@ -94,7 +96,7 @@ test_that("Table with metadata roundtrip", {
   skip_if_no_pyarrow()
   tab <- Table$create(example_with_times)
   pytab <- reticulate::r_to_py(tab)
-  expect_is(pytab, "pyarrow.lib.Table")
+  expect_s3_class(pytab, "pyarrow.lib.Table")
   expect_equal(reticulate::py_to_r(pytab), tab)
   expect_identical(as.data.frame(reticulate::py_to_r(pytab)), example_with_times)
 })
